@@ -30,6 +30,7 @@ import tempfile
 import traceback
 from types import TracebackType
 from typing import Any
+from typing import cast
 from typing import IO
 from typing import Optional
 from typing import Tuple
@@ -40,6 +41,7 @@ from certbot import configuration
 from certbot import errors
 from certbot import util
 from certbot._internal import constants
+from certbot._internal.display import util as display_util
 from certbot.compat import os
 
 # Logging format
@@ -73,7 +75,10 @@ def pre_arg_parse_setup() -> None:
 
     stream_handler = ColoredStreamHandler()
     stream_handler.setFormatter(logging.Formatter(CLI_FMT))
-    stream_handler.setLevel(constants.QUIET_LOGGING_LEVEL)
+    # The pre-argparse logging level is set to WARNING here. This is to ensure that
+    # deprecated flags (see DeprecatedArgumentAction) print something to the terminal.
+    # See https://github.com/certbot/certbot/issues/9618.
+    stream_handler.setLevel(logging.WARNING)
 
     root_logger = logging.getLogger()
     root_logger.setLevel(logging.DEBUG)  # send all records to handlers
@@ -162,7 +167,7 @@ def setup_log_file_handler(config: configuration.NamespaceConfig, logfile: str,
         handler = logging.handlers.RotatingFileHandler(
             log_file_path, maxBytes=2 ** 20,
             backupCount=config.max_log_backups)
-    except IOError as error:
+    except OSError as error:
         raise errors.Error(util.PERM_ERR_FMT.format(error))
     # rotate on each invocation, rollover only possible when maxBytes
     # is nonzero and backupCount is nonzero, so we set maxBytes as big
@@ -262,7 +267,7 @@ class TempHandler(logging.StreamHandler):
 
     """
     def __init__(self) -> None:
-        self._workdir = tempfile.mkdtemp()
+        self._workdir = tempfile.mkdtemp(prefix="certbot-log-")
         self.path = os.path.join(self._workdir, 'log')
         stream = util.safe_open(self.path, mode='w', chmod=0o600)
         super().__init__(stream)
@@ -346,7 +351,11 @@ def post_arg_parse_except_hook(exc_type: Type[BaseException], exc_value: BaseExc
     """
     exc_info = (exc_type, exc_value, trace)
     # Only print human advice if not running under --quiet
-    exit_func = lambda: sys.exit(1) if quiet else exit_with_advice(log_path)
+    def exit_func() -> None:
+        if quiet:
+            sys.exit(1)
+        else:
+            exit_with_advice(log_path)
     # constants.QUIET_LOGGING_LEVEL or higher should be used to
     # display message the user, otherwise, a lower level like
     # logger.DEBUG should be used
@@ -365,9 +374,7 @@ def post_arg_parse_except_hook(exc_type: Type[BaseException], exc_value: BaseExc
             exit_func()
         logger.error('An unexpected error occurred:')
         if messages.is_acme_error(exc_value):
-            # Remove the ACME error prefix from the exception
-            _, _, exc_str = str(exc_value).partition(':: ')
-            logger.error(exc_str)
+            logger.error(display_util.describe_acme_error(cast(messages.Error, exc_value)))
         else:
             output = traceback.format_exception_only(exc_type, exc_value)
             # format_exception_only returns a list of strings each
