@@ -140,8 +140,6 @@ def _get_and_save_cert(le_client: client.Client, config: configuration.Namespace
                 )
             )
             lineage = le_client.obtain_and_enroll_certificate(domains, certname)
-            if lineage is False:
-                raise errors.Error("Certificate could not be obtained")
             if lineage is not None:
                 hooks.deploy_hook(config, lineage.names(), lineage.live_dir)
                 renewed_domains.extend(domains)
@@ -711,8 +709,8 @@ def _determine_account(config: configuration.NamespaceConfig
     def _tos_cb(terms_of_service: str) -> None:
         if config.tos:
             return
-        msg = ("Please read the Terms of Service at {0}. You "
-               "must agree in order to register with the ACME "
+        msg = ("Please read the Terms of Service at: {0}\n"
+               "You must agree in order to register with the ACME "
                "server. Do you agree?".format(terms_of_service))
         result = display_util.yesno(msg, cli_flag="--agree-tos", force_interactive=True)
         if not result:
@@ -934,7 +932,7 @@ def update_account(config: configuration.NamespaceConfig,
     if not accounts:
         return f"Could not find an existing account for server {config.server}."
     if config.email is None and not config.register_unsafely_without_email:
-        config.email = display_ops.get_email(optional=False)
+        config.email = display_ops.get_email()
 
     acc, acme = _determine_account(config)
     cb_client = client.Client(config, acc, None, None, acme=acme)
