@@ -4,40 +4,59 @@
     is capable of handling the signatures.
 
 """
+import datetime
 import hashlib
 import logging
+import re
+from typing import List
+from typing import Optional
+from typing import Set
+from typing import Tuple
+from typing import TYPE_CHECKING
+from typing import Union
 import warnings
 
-import re
-# See https://github.com/pyca/cryptography/issues/4275
-from cryptography import x509  # type: ignore
-from cryptography.exceptions import InvalidSignature, UnsupportedAlgorithm
+from cryptography import x509
+from cryptography.exceptions import InvalidSignature
+from cryptography.exceptions import UnsupportedAlgorithm
 from cryptography.hazmat.backends import default_backend
+from cryptography.hazmat.primitives import hashes
+from cryptography.hazmat.primitives import serialization
 from cryptography.hazmat.primitives.asymmetric import ec
-from cryptography.hazmat.primitives.asymmetric.ec import ECDSA, EllipticCurvePublicKey
+from cryptography.hazmat.primitives.asymmetric import rsa
+from cryptography.hazmat.primitives.asymmetric.dsa import DSAPublicKey
+from cryptography.hazmat.primitives.asymmetric.ec import ECDSA
+from cryptography.hazmat.primitives.asymmetric.ec import EllipticCurvePublicKey
 from cryptography.hazmat.primitives.asymmetric.padding import PKCS1v15
 from cryptography.hazmat.primitives.asymmetric.rsa import RSAPublicKey
-from cryptography.hazmat.primitives.serialization import Encoding, NoEncryption, PrivateFormat
+from cryptography.hazmat.primitives.serialization import Encoding
+from cryptography.hazmat.primitives.serialization import NoEncryption
+from cryptography.hazmat.primitives.serialization import PrivateFormat
+import josepy
 from OpenSSL import crypto
-from OpenSSL import SSL  # type: ignore
-
-import pyrfc3339
-import zope.component
+from OpenSSL import SSL
 
 from acme import crypto_util as acme_crypto_util
-from acme.magic_typing import IO  # pylint: disable=unused-import
 from certbot import errors
 from certbot import interfaces
 from certbot import util
 from certbot.compat import os
 
+# Cryptography ed448 and ed25519 modules do not exist on oldest tests
+if TYPE_CHECKING:
+    from cryptography.hazmat.primitives.asymmetric.ed448 import Ed448PublicKey
+    from cryptography.hazmat.primitives.asymmetric.ed25519 import Ed25519PublicKey
+    from cryptography.hazmat.primitives.asymmetric.x448 import X448PublicKey
+    from cryptography.hazmat.primitives.asymmetric.x25519 import X25519PublicKey
+
 logger = logging.getLogger(__name__)
 
 
 # High level functions
-def init_save_key(
-        key_size, key_dir, key_type="rsa", elliptic_curve="secp256r1", keyname="key-certbot.pem"
-):
+
+def generate_key(key_size: int, key_dir: Optional[str], key_type: str = "rsa",
+                 elliptic_curve: str = "secp256r1", keyname: str = "key-certbot.pem",
+                 strict_permissions: bool = True) -> util.Key:
     """Initializes and saves a privkey.
 
     Inits key and saves it in PEM format on the filesystem.
@@ -46,10 +65,12 @@ def init_save_key(
         already exists at the path.
 
     :param int key_size: key size in bits if key size is rsa.
-    :param str key_dir: Key save directory.
+    :param str key_dir: Optional key save directory.
     :param str key_type: Key Type [rsa, ecdsa]
     :param str elliptic_curve: Name of the elliptic curve if key type is ecdsa.
     :param str keyname: Filename of key
+    :param bool strict_permissions: If true and key_dir exists, an exception is raised if
+        the directory doesn't have 0700 permissions or isn't owned by the current user.
 
     :returns: Key
     :rtype: :class:`certbot.util.Key`
@@ -62,50 +83,54 @@ def init_save_key(
             bits=key_size, elliptic_curve=elliptic_curve or "secp256r1", key_type=key_type,
         )
     except ValueError as err:
-        logger.error("", exc_info=True)
+        logger.debug("", exc_info=True)
+        logger.error("Encountered error while making key: %s", str(err))
         raise err
 
-    config = zope.component.getUtility(interfaces.IConfig)
     # Save file
-    util.make_or_verify_dir(key_dir, 0o700, config.strict_permissions)
-    key_f, key_path = util.unique_file(
-        os.path.join(key_dir, keyname), 0o600, "wb")
-    with key_f:
-        key_f.write(key_pem)
-    if key_type == 'rsa':
-        logger.debug("Generating RSA key (%d bits): %s", key_size, key_path)
-    else:
-        logger.debug("Generating ECDSA key (%d bits): %s", key_size, key_path)
+    key_path = None
+    if key_dir:
+        util.make_or_verify_dir(key_dir, 0o700, strict_permissions)
+        key_f, key_path = util.unique_file(
+            os.path.join(key_dir, keyname), 0o600, "wb")
+        with key_f:
+            key_f.write(key_pem)
+        if key_type == 'rsa':
+            logger.debug("Generating RSA key (%d bits): %s", key_size, key_path)
+        else:
+            logger.debug("Generating ECDSA key (%d bits): %s", key_size, key_path)
 
     return util.Key(key_path, key_pem)
 
 
-def init_save_csr(privkey, names, path):
+def generate_csr(privkey: util.Key, names: Union[List[str], Set[str]], path: Optional[str],
+                 must_staple: bool = False, strict_permissions: bool = True) -> util.CSR:
     """Initialize a CSR with the given private key.
 
     :param privkey: Key to include in the CSR
     :type privkey: :class:`certbot.util.Key`
-
     :param set names: `str` names to include in the CSR
-
-    :param str path: Certificate save directory.
+    :param str path: Optional certificate save directory.
+    :param bool must_staple: If true, include the TLS Feature extension "OCSP Must-Staple"
+    :param bool strict_permissions: If true and path exists, an exception is raised if
+        the directory doesn't have 0755 permissions or isn't owned by the current user.
 
     :returns: CSR
     :rtype: :class:`certbot.util.CSR`
 
     """
-    config = zope.component.getUtility(interfaces.IConfig)
-
     csr_pem = acme_crypto_util.make_csr(
-        privkey.pem, names, must_staple=config.must_staple)
+        privkey.pem, names, must_staple=must_staple)
 
-    # Save CSR
-    util.make_or_verify_dir(path, 0o755, config.strict_permissions)
-    csr_f, csr_filename = util.unique_file(
-        os.path.join(path, "csr-certbot.pem"), 0o644, "wb")
-    with csr_f:
-        csr_f.write(csr_pem)
-    logger.debug("Creating CSR: %s", csr_filename)
+    # Save CSR, if requested
+    csr_filename = None
+    if path:
+        util.make_or_verify_dir(path, 0o755, strict_permissions)
+        csr_f, csr_filename = util.unique_file(
+            os.path.join(path, "csr-certbot.pem"), 0o644, "wb")
+        with csr_f:
+            csr_f.write(csr_pem)
+        logger.debug("Creating CSR: %s", csr_filename)
 
     return util.CSR(csr_filename, csr_pem, "pem")
 
@@ -115,98 +140,103 @@ def init_save_csr(privkey, names, path):
 # A. Do more checks to verify that the CSR is trusted/valid
 # B. Audit the parsing code for vulnerabilities
 
-def valid_csr(csr):
+def valid_csr(csr: bytes) -> bool:
     """Validate CSR.
 
-    Check if `csr` is a valid CSR for the given domains.
+    Check if `csr` is a valid CSR with a correct self-signed signature.
 
-    :param str csr: CSR in PEM.
+    :param bytes csr: CSR in PEM.
 
     :returns: Validity of CSR.
     :rtype: bool
 
     """
     try:
-        req = crypto.load_certificate_request(
-            crypto.FILETYPE_PEM, csr)
-        return req.verify(req.get_pubkey())
-    except crypto.Error:
+        req = x509.load_pem_x509_csr(csr)
+        return req.is_signature_valid
+    except (ValueError, TypeError):
         logger.debug("", exc_info=True)
         return False
 
 
-def csr_matches_pubkey(csr, privkey):
+def csr_matches_pubkey(csr: bytes, privkey: bytes) -> bool:
     """Does private key correspond to the subject public key in the CSR?
 
-    :param str csr: CSR in PEM.
-    :param str privkey: Private key file contents (PEM)
+    :param bytes csr: CSR in PEM.
+    :param bytes privkey: Private key file contents (PEM)
 
     :returns: Correspondence of private key to CSR subject public key.
     :rtype: bool
 
     """
-    req = crypto.load_certificate_request(
-        crypto.FILETYPE_PEM, csr)
-    pkey = crypto.load_privatekey(crypto.FILETYPE_PEM, privkey)
-    try:
-        return req.verify(pkey)
-    except crypto.Error:
-        logger.debug("", exc_info=True)
-        return False
+    req = x509.load_pem_x509_csr(csr)
+    pkey = serialization.load_pem_private_key(privkey, password=None)
+    return req.is_signature_valid and req.public_key() == pkey.public_key()
 
 
-def import_csr_file(csrfile, data):
+def import_csr_file(
+    csrfile: str, data: bytes
+) -> Tuple[acme_crypto_util.Format, util.CSR, List[str]]:
     """Import a CSR file, which can be either PEM or DER.
 
     :param str csrfile: CSR filename
-    :param str data: contents of the CSR file
+    :param bytes data: contents of the CSR file
 
-    :returns: (`crypto.FILETYPE_PEM`,
+    :returns: (`acme_crypto_util.Format.PEM`,
                util.CSR object representing the CSR,
                list of domains requested in the CSR)
     :rtype: tuple
 
     """
-    PEM = crypto.FILETYPE_PEM
-    load = crypto.load_certificate_request
     try:
         # Try to parse as DER first, then fall back to PEM.
-        csr = load(crypto.FILETYPE_ASN1, data)
-    except crypto.Error:
+        csr = x509.load_der_x509_csr(data)
+    except ValueError:
         try:
-            csr = load(PEM, data)
-        except crypto.Error:
+            csr = x509.load_pem_x509_csr(data)
+        except ValueError:
             raise errors.Error("Failed to parse CSR file: {0}".format(csrfile))
 
-    domains = _get_names_from_loaded_cert_or_req(csr)
+    domains = acme_crypto_util.get_names_from_subject_and_extensions(csr.subject, csr.extensions)
     # Internally we always use PEM, so re-encode as PEM before returning.
-    data_pem = crypto.dump_certificate_request(PEM, csr)
-    return PEM, util.CSR(file=csrfile, data=data_pem, form="pem"), domains
+    data_pem = csr.public_bytes(serialization.Encoding.PEM)
+    return (
+        acme_crypto_util.Format.PEM,
+        util.CSR(file=csrfile, data=data_pem, form="pem"),
+        domains,
+    )
 
 
-def make_key(bits=1024, key_type="rsa", elliptic_curve=None):
+def make_key(bits: int = 2048, key_type: str = "rsa",
+             elliptic_curve: Optional[str] = None) -> bytes:
     """Generate PEM encoded RSA|EC key.
 
-    :param int bits: Number of bits if key_type=rsa. At least 1024 for RSA.
-
-    :param str ec_curve: The elliptic curve to use.
+    :param int bits: Number of bits if key_type=rsa. At least 2048 for RSA.
+    :param str key_type: The type of key to generate, but be rsa or ecdsa
+    :param str elliptic_curve: The elliptic curve to use.
 
     :returns: new RSA or ECDSA key in PEM form with specified number of bits
               or of type ec_curve when key_type ecdsa is used.
-    :rtype: str
+    :rtype: bytes
+
     """
+    key: Union[rsa.RSAPrivateKey, ec.EllipticCurvePrivateKey]
     if key_type == 'rsa':
-        if bits < 1024:
+        if bits < 2048:
             raise errors.Error("Unsupported RSA key length: {}".format(bits))
 
-        key = crypto.PKey()
-        key.generate_key(crypto.TYPE_RSA, bits)
+        key = rsa.generate_private_key(public_exponent=65537, key_size=bits)
     elif key_type == 'ecdsa':
+        if not elliptic_curve:
+            raise errors.Error("When key_type == ecdsa, elliptic_curve must be set.")
         try:
             name = elliptic_curve.upper()
             if name in ('SECP256R1', 'SECP384R1', 'SECP521R1'):
-                _key = ec.generate_private_key(
-                    curve=getattr(ec, elliptic_curve.upper(), None)(),
+                curve = getattr(ec, elliptic_curve.upper())
+                if not curve:
+                    raise errors.Error(f"Invalid curve type: {elliptic_curve}")
+                key = ec.generate_private_key(
+                    curve=curve(),
                     backend=default_backend()
                 )
             else:
@@ -215,34 +245,35 @@ def make_key(bits=1024, key_type="rsa", elliptic_curve=None):
             raise errors.Error("Unsupported elliptic curve: {}".format(elliptic_curve))
         except UnsupportedAlgorithm as e:
             raise e from errors.Error(str(e))
-        _key_pem = _key.private_bytes(
-            encoding=Encoding.PEM,
-            format=PrivateFormat.TraditionalOpenSSL,
-            encryption_algorithm=NoEncryption()
-        )
-        key = crypto.load_privatekey(crypto.FILETYPE_PEM, _key_pem)
     else:
         raise errors.Error("Invalid key_type specified: {}.  Use [rsa|ecdsa]".format(key_type))
-    return crypto.dump_privatekey(crypto.FILETYPE_PEM, key)
+    return key.private_bytes(
+        encoding=Encoding.PEM,
+        format=PrivateFormat.PKCS8,
+        encryption_algorithm=NoEncryption()
+    )
 
 
-def valid_privkey(privkey):
+def valid_privkey(privkey: Union[str, bytes]) -> bool:
     """Is valid RSA private key?
 
-    :param str privkey: Private key file contents in PEM
+    :param privkey: Private key file contents in PEM
 
     :returns: Validity of private key.
     :rtype: bool
 
     """
+    if isinstance(privkey, str):
+        privkey = privkey.encode()
     try:
-        return crypto.load_privatekey(
-            crypto.FILETYPE_PEM, privkey).check()
-    except (TypeError, crypto.Error):
+        serialization.load_pem_private_key(privkey, password=None)
+    except ValueError:
         return False
+    else:
+        return True
 
 
-def verify_renewable_cert(renewable_cert):
+def verify_renewable_cert(renewable_cert: interfaces.RenewableCert) -> None:
     """For checking that your certs were not corrupted on disk.
 
     Several things are checked:
@@ -260,7 +291,7 @@ def verify_renewable_cert(renewable_cert):
     verify_cert_matches_priv_key(renewable_cert.cert_path, renewable_cert.key_path)
 
 
-def verify_renewable_cert_sig(renewable_cert):
+def verify_renewable_cert_sig(renewable_cert: interfaces.RenewableCert) -> None:
     """Verifies the signature of a RenewableCert object.
 
     :param renewable_cert: cert to verify
@@ -269,53 +300,49 @@ def verify_renewable_cert_sig(renewable_cert):
     :raises errors.Error: If signature verification fails.
     """
     try:
-        with open(renewable_cert.chain_path, 'rb') as chain_file:  # type: IO[bytes]
+        with open(renewable_cert.chain_path, 'rb') as chain_file:
             chain = x509.load_pem_x509_certificate(chain_file.read(), default_backend())
-        with open(renewable_cert.cert_path, 'rb') as cert_file:  # type: IO[bytes]
+        with open(renewable_cert.cert_path, 'rb') as cert_file:
             cert = x509.load_pem_x509_certificate(cert_file.read(), default_backend())
         pk = chain.public_key()
-        with warnings.catch_warnings():
-            verify_signed_payload(pk, cert.signature, cert.tbs_certificate_bytes,
-                                  cert.signature_hash_algorithm)
-    except (IOError, ValueError, InvalidSignature) as e:
+        assert cert.signature_hash_algorithm # always present for RSA and ECDSA
+        verify_signed_payload(pk, cert.signature, cert.tbs_certificate_bytes,
+                                cert.signature_hash_algorithm)
+    except (OSError, ValueError, InvalidSignature) as e:
         error_str = "verifying the signature of the certificate located at {0} has failed. \
                 Details: {1}".format(renewable_cert.cert_path, e)
         logger.exception(error_str)
         raise errors.Error(error_str)
 
 
-def verify_signed_payload(public_key, signature, payload, signature_hash_algorithm):
+def verify_signed_payload(public_key: Union[DSAPublicKey, 'Ed25519PublicKey', 'Ed448PublicKey',
+                                            EllipticCurvePublicKey, RSAPublicKey,
+                                            'X25519PublicKey', 'X448PublicKey'],
+                          signature: bytes, payload: bytes,
+                          signature_hash_algorithm: hashes.HashAlgorithm) -> None:
     """Check the signature of a payload.
 
     :param RSAPublicKey/EllipticCurvePublicKey public_key: the public_key to check signature
     :param bytes signature: the signature bytes
     :param bytes payload: the payload bytes
-    :param cryptography.hazmat.primitives.hashes.HashAlgorithm \
-           signature_hash_algorithm: algorithm used to hash the payload
+    :param hashes.HashAlgorithm signature_hash_algorithm: algorithm used to hash the payload
 
     :raises InvalidSignature: If signature verification fails.
     :raises errors.Error: If public key type is not supported
     """
-    with warnings.catch_warnings():
-        warnings.simplefilter("ignore")
-        if isinstance(public_key, RSAPublicKey):
-            # https://github.com/python/typeshed/blob/master/third_party/2/cryptography/hazmat/primitives/asymmetric/rsa.pyi
-            verifier = public_key.verifier(  # type: ignore
-                signature, PKCS1v15(), signature_hash_algorithm
-            )
-            verifier.update(payload)
-            verifier.verify()
-        elif isinstance(public_key, EllipticCurvePublicKey):
-            verifier = public_key.verifier(
-                signature, ECDSA(signature_hash_algorithm)
-            )
-            verifier.update(payload)
-            verifier.verify()
-        else:
-            raise errors.Error("Unsupported public key type")
+    if isinstance(public_key, RSAPublicKey):
+        public_key.verify(
+            signature, payload, PKCS1v15(), signature_hash_algorithm
+        )
+    elif isinstance(public_key, EllipticCurvePublicKey):
+        public_key.verify(
+            signature, payload, ECDSA(signature_hash_algorithm)
+        )
+    else:
+        raise errors.Error("Unsupported public key type.")
 
 
-def verify_cert_matches_priv_key(cert_path, key_path):
+def verify_cert_matches_priv_key(cert_path: str, key_path: str) -> None:
     """ Verifies that the private key and cert match.
 
     :param str cert_path: path to a cert in PEM format
@@ -328,7 +355,7 @@ def verify_cert_matches_priv_key(cert_path, key_path):
         context.use_certificate_file(cert_path)
         context.use_privatekey_file(key_path)
         context.check_privatekey()
-    except (IOError, SSL.Error) as e:
+    except (OSError, SSL.Error) as e:
         error_str = "verifying the certificate located at {0} matches the \
                 private key located at {1} has failed. \
                 Details: {2}".format(cert_path,
@@ -337,7 +364,7 @@ def verify_cert_matches_priv_key(cert_path, key_path):
         raise errors.Error(error_str)
 
 
-def verify_fullchain(renewable_cert):
+def verify_fullchain(renewable_cert: interfaces.RenewableCert) -> None:
     """ Verifies that fullchain is indeed cert concatenated with chain.
 
     :param renewable_cert: cert to verify
@@ -346,17 +373,17 @@ def verify_fullchain(renewable_cert):
     :raises errors.Error: If cert and chain do not combine to fullchain.
     """
     try:
-        with open(renewable_cert.chain_path) as chain_file:  # type: IO[str]
+        with open(renewable_cert.chain_path) as chain_file:
             chain = chain_file.read()
-        with open(renewable_cert.cert_path) as cert_file:  # type: IO[str]
+        with open(renewable_cert.cert_path) as cert_file:
             cert = cert_file.read()
-        with open(renewable_cert.fullchain_path) as fullchain_file:  # type: IO[str]
+        with open(renewable_cert.fullchain_path) as fullchain_file:
             fullchain = fullchain_file.read()
         if (cert + chain) != fullchain:
             error_str = "fullchain does not match cert + chain for {0}!"
             error_str = error_str.format(renewable_cert.lineagename)
             raise errors.Error(error_str)
-    except IOError as e:
+    except OSError as e:
         error_str = "reading one of cert, chain, or fullchain has failed: {0}".format(e)
         logger.exception(error_str)
         raise errors.Error(error_str)
@@ -364,12 +391,20 @@ def verify_fullchain(renewable_cert):
         raise e
 
 
-def pyopenssl_load_certificate(data):
+def pyopenssl_load_certificate(data: bytes) -> Tuple[crypto.X509, int]:
     """Load PEM/DER certificate.
 
     :raises errors.Error:
 
+    Deprecated
+    .. deprecated: 3.2.1
     """
+    warnings.warn(
+        "certbot.crypto_util.pyopenssl_load_certificate is deprecated and "
+        "will be removed in the next major release of Certbot.",
+        DeprecationWarning,
+        stacklevel=2
+    )
 
     openssl_errors = []
 
@@ -382,73 +417,104 @@ def pyopenssl_load_certificate(data):
         str(error) for error in openssl_errors)))
 
 
-def _load_cert_or_req(cert_or_req_str, load_func,
-                      typ=crypto.FILETYPE_PEM):
-    try:
-        return load_func(typ, cert_or_req_str)
-    except crypto.Error:
-        logger.error("", exc_info=True)
-        raise
-
-
-def _get_sans_from_cert_or_req(cert_or_req_str, load_func,
-                               typ=crypto.FILETYPE_PEM):
-    # pylint: disable=protected-access
-    return acme_crypto_util._pyopenssl_cert_or_req_san(_load_cert_or_req(
-        cert_or_req_str, load_func, typ))
-
-
-def get_sans_from_cert(cert, typ=crypto.FILETYPE_PEM):
+def get_sans_from_cert(
+    cert: bytes, typ: Union[acme_crypto_util.Format, int] = acme_crypto_util.Format.PEM
+) -> List[str]:
     """Get a list of Subject Alternative Names from a certificate.
 
     :param str cert: Certificate (encoded).
-    :param typ: `crypto.FILETYPE_PEM` or `crypto.FILETYPE_ASN1`
+    :param Format typ: Which format the `cert` bytes are in.
 
     :returns: A list of Subject Alternative Names.
     :rtype: list
 
     """
-    return _get_sans_from_cert_or_req(
-        cert, crypto.load_certificate, typ)
+    typ = acme_crypto_util.Format(typ)
+    if typ == acme_crypto_util.Format.PEM:
+        x509_cert = x509.load_pem_x509_certificate(cert)
+    else:
+        assert typ == acme_crypto_util.Format.DER
+        x509_cert = x509.load_der_x509_certificate(cert)
+
+    try:
+        san_ext = x509_cert.extensions.get_extension_for_class(
+            x509.SubjectAlternativeName
+        )
+    except x509.ExtensionNotFound:
+        return []
+
+    return san_ext.value.get_values_for_type(x509.DNSName)
 
 
-def _get_names_from_cert_or_req(cert_or_req, load_func, typ):
-    loaded_cert_or_req = _load_cert_or_req(cert_or_req, load_func, typ)
-    return _get_names_from_loaded_cert_or_req(loaded_cert_or_req)
-
-
-def _get_names_from_loaded_cert_or_req(loaded_cert_or_req):
-    # pylint: disable=protected-access
-    return acme_crypto_util._pyopenssl_cert_or_req_all_names(loaded_cert_or_req)
-
-
-def get_names_from_cert(csr, typ=crypto.FILETYPE_PEM):
+def get_names_from_cert(
+    cert: bytes, typ: Union[acme_crypto_util.Format, int] = acme_crypto_util.Format.PEM
+) -> List[str]:
     """Get a list of domains from a cert, including the CN if it is set.
 
     :param str cert: Certificate (encoded).
-    :param typ: `crypto.FILETYPE_PEM` or `crypto.FILETYPE_ASN1`
+    :param Format typ: Which format the `cert` bytes are in.
 
     :returns: A list of domain names.
     :rtype: list
 
     """
-    return _get_names_from_cert_or_req(
-        csr, crypto.load_certificate, typ)
+    typ = acme_crypto_util.Format(typ)
+    if typ == acme_crypto_util.Format.PEM:
+        x509_cert = x509.load_pem_x509_certificate(cert)
+    else:
+        assert typ == acme_crypto_util.Format.DER
+        x509_cert = x509.load_der_x509_certificate(cert)
+    return acme_crypto_util.get_names_from_subject_and_extensions(
+        x509_cert.subject, x509_cert.extensions
+    )
 
 
-def dump_pyopenssl_chain(chain, filetype=crypto.FILETYPE_PEM):
+def get_names_from_req(
+    csr: bytes, typ: Union[acme_crypto_util.Format, int] = acme_crypto_util.Format.PEM
+) -> List[str]:
+    """Get a list of domains from a CSR, including the CN if it is set.
+
+    :param str csr: CSR (encoded).
+    :param acme_crypto_util.Format typ: Which format the `csr` bytes are in.
+    :returns: A list of domain names.
+    :rtype: list
+
+    """
+    typ = acme_crypto_util.Format(typ)
+    if typ == acme_crypto_util.Format.PEM:
+        x509_req = x509.load_pem_x509_csr(csr)
+    else:
+        assert typ == acme_crypto_util.Format.DER
+        x509_req = x509.load_der_x509_csr(csr)
+    return acme_crypto_util.get_names_from_subject_and_extensions(
+        x509_req.subject, x509_req.extensions
+    )
+
+
+def dump_pyopenssl_chain(
+    chain: Union[List[crypto.X509], List[josepy.ComparableX509]],
+    filetype: Union[acme_crypto_util.Format, int] = acme_crypto_util.Format.PEM,
+) -> bytes:
     """Dump certificate chain into a bundle.
 
     :param list chain: List of `crypto.X509` (or wrapped in
         :class:`josepy.util.ComparableX509`).
 
+    Deprecated
+    .. deprecated: 3.2.1
     """
+    warnings.warn(
+        "certbot.crypto_util.dump_pyopenssl_chain is deprecated and "
+        "will be removed in the next major release of Certbot.",
+        DeprecationWarning,
+        stacklevel=2
+    )
     # XXX: returns empty string when no chain is available, which
     # shuts up RenewableCert, but might not be the best solution...
     return acme_crypto_util.dump_pyopenssl_chain(chain, filetype)
 
 
-def notBefore(cert_path):
+def notBefore(cert_path: str) -> datetime.datetime:
     """When does the cert at cert_path start being valid?
 
     :param str cert_path: path to a cert in PEM format
@@ -457,10 +523,12 @@ def notBefore(cert_path):
     :rtype: :class:`datetime.datetime`
 
     """
-    return _notAfterBefore(cert_path, crypto.X509.get_notBefore)
+    with open(cert_path, "rb") as f:
+        cert = x509.load_pem_x509_certificate(f.read())
+    return cert.not_valid_before_utc
 
 
-def notAfter(cert_path):
+def notAfter(cert_path: str) -> datetime.datetime:
     """When does the cert at cert_path stop being valid?
 
     :param str cert_path: path to a cert in PEM format
@@ -469,35 +537,12 @@ def notAfter(cert_path):
     :rtype: :class:`datetime.datetime`
 
     """
-    return _notAfterBefore(cert_path, crypto.X509.get_notAfter)
+    with open(cert_path, "rb") as f:
+        cert = x509.load_pem_x509_certificate(f.read())
+    return cert.not_valid_after_utc
 
 
-def _notAfterBefore(cert_path, method):
-    """Internal helper function for finding notbefore/notafter.
-
-    :param str cert_path: path to a cert in PEM format
-    :param function method: one of ``crypto.X509.get_notBefore``
-        or ``crypto.X509.get_notAfter``
-
-    :returns: the notBefore or notAfter value from the cert at cert_path
-    :rtype: :class:`datetime.datetime`
-
-    """
-    # pylint: disable=redefined-outer-name
-    with open(cert_path, "rb") as f:  # type: IO[bytes]
-        x509 = crypto.load_certificate(crypto.FILETYPE_PEM, f.read())
-    # pyopenssl always returns bytes
-    timestamp = method(x509)
-    reformatted_timestamp = [timestamp[0:4], b"-", timestamp[4:6], b"-",
-                             timestamp[6:8], b"T", timestamp[8:10], b":",
-                             timestamp[10:12], b":", timestamp[12:]]
-    # pyrfc3339 always uses the type `str`
-    timestamp_bytes = b"".join(reformatted_timestamp)
-    timestamp_str = timestamp_bytes.decode('ascii')
-    return pyrfc3339.parse(timestamp_str)
-
-
-def sha256sum(filename):
+def sha256sum(filename: str) -> str:
     """Compute a sha256sum of a file.
 
     NB: In given file, platform specific newlines characters will be converted
@@ -524,7 +569,7 @@ CERT_PEM_REGEX = re.compile(
 )
 
 
-def cert_and_chain_from_fullchain(fullchain_pem):
+def cert_and_chain_from_fullchain(fullchain_pem: str) -> Tuple[str, str]:
     """Split fullchain_pem into cert_pem and chain_pem
 
     :param str fullchain_pem: concatenated cert + chain
@@ -543,16 +588,19 @@ def cert_and_chain_from_fullchain(fullchain_pem):
         raise errors.Error("failed to parse fullchain into cert and chain: " +
                            "less than 2 certificates in chain")
 
-    # Second pass: for each certificate found, parse it using OpenSSL and re-encode it,
+    # Second pass: for each certificate found, parse it using cryptography and re-encode it,
     # with the effect of normalizing any encoding variations (e.g. CRLF, whitespace).
-    certs_normalized = [crypto.dump_certificate(crypto.FILETYPE_PEM,
-        crypto.load_certificate(crypto.FILETYPE_PEM, cert)).decode() for cert in certs]
+    certs_normalized: List[str] = []
+    for cert_pem in certs:
+        cert = x509.load_pem_x509_certificate(cert_pem)
+        cert_pem = cert.public_bytes(Encoding.PEM)
+        certs_normalized.append(cert_pem.decode())
 
     # Since each normalized cert has a newline suffix, no extra newlines are required.
     return (certs_normalized[0], "".join(certs_normalized[1:]))
 
 
-def get_serial_from_cert(cert_path):
+def get_serial_from_cert(cert_path: str) -> int:
     """Retrieve the serial number of a certificate from certificate path
 
     :param str cert_path: path to a cert in PEM format
@@ -560,13 +608,13 @@ def get_serial_from_cert(cert_path):
     :returns: serial number of the certificate
     :rtype: int
     """
-    # pylint: disable=redefined-outer-name
-    with open(cert_path, "rb") as f:  # type: IO[bytes]
-        x509 = crypto.load_certificate(crypto.FILETYPE_PEM, f.read())
-    return x509.get_serial_number()
+    with open(cert_path, "rb") as f:
+        cert = x509.load_pem_x509_certificate(f.read())
+    return cert.serial_number
 
 
-def find_chain_with_issuer(fullchains, issuer_cn, warn_on_no_match=False):
+def find_chain_with_issuer(fullchains: List[str], issuer_cn: str,
+                           warn_on_no_match: bool = False) -> str:
     """Chooses the first certificate chain from fullchains whose topmost
     intermediate has an Issuer Common Name matching issuer_cn (in other words
     the first chain which chains to a root whose name matches issuer_cn).
@@ -588,7 +636,7 @@ def find_chain_with_issuer(fullchains, issuer_cn, warn_on_no_match=False):
 
     # Nothing matched, return whatever was first in the list.
     if warn_on_no_match:
-        logger.info("Certbot has been configured to prefer certificate chains with "
+        logger.warning("Certbot has been configured to prefer certificate chains with "
                     "issuer '%s', but no chain from the CA matched this issuer. Using "
                     "the default certificate chain instead.", issuer_cn)
     return fullchains[0]
